@@ -1,4 +1,4 @@
-/************************************************************************
+/*
  *   IRC - Internet Relay Chat, ircd/s_serv.c (formerly ircd/s_msg.c)
  *   Copyright (C) 1990 Jarkko Oikarinen and
  *                      University of Oulu, Computing Center
@@ -21,11 +21,6 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef lint
-static  char sccsid[] = "@(#)s_serv.c	2.55 2/7/94 (C) 1988 University of Oulu, \
-Computing Center and Jarkko Oikarinen";
-#endif
-
 #include "struct.h"
 #include "common.h"
 #include "sys.h"
@@ -33,18 +28,17 @@ Computing Center and Jarkko Oikarinen";
 #include "msg.h"
 #include "channel.h"
 #include "userload.h"
-#if defined(PCS) || defined(AIX) || defined(DYNIXPTX) || defined(SVR3)
-#include <time.h>
-#endif
 #include <sys/stat.h>
 #include <fcntl.h>
-#ifndef _WIN32
-#include <utmp.h>
-#else
-#include <io.h>
-#endif
 #include "h.h"
 
+#include "ircd/match.h"
+#include "ircd/send.h"
+#include "ircd/string.h"
+
+IRCD_SCCSID("@(#)s_serv.c	2.55 2/7/94 (C) 1988 University of Oulu, Computing Center and Jarkko Oikarinen");
+IRCD_RCSID("$Id: s_serv.c,v 1.1.1.8.2.16 2005/01/12 18:13:33 onno Exp $");
+
 static	char	buf[BUFSIZE];
 extern int oper_access[];
 
@@ -112,7 +106,8 @@ int     max_connection_count = 1, max_client_count = 1;
 **	parv[0] = sender prefix
 **	parv[1] = remote server
 */
-int	m_version(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_version(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	extern	char	serveropts[];
 
@@ -121,11 +116,7 @@ int	m_version(aClient *cptr, aClient *sptr, int parc, char *parv[])
 
 	if (hunt_server(cptr,sptr,":%s VERSION :%s",1,parc,parv)==HUNTED_ISME)
 		sendto_one(sptr, rpl_str(RPL_VERSION), me.name,
-			   parv[0], version, debugmode, me.name, serveropts
-#ifdef USE_CASETABLES
-			   , casetable
-#endif
-			   );
+			   parv[0], version, debugmode, me.name, serveropts);
 	return 0;
 }
 
@@ -135,7 +126,8 @@ int	m_version(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[1] = server name
 **	parv[parc-1] = comment
 */
-int	m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	aConfItem *aconf;
 	char	*server;
@@ -272,7 +264,8 @@ int	m_squit(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[2] = serverinfo/hopcount
 **      parv[3] = serverinfo
 */
-int	m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	char	*ch;
 	int	i;
@@ -364,10 +357,7 @@ int	m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	  if(*cptr->passwd)
 	    {
 	      char    salt[3];
-#ifndef USE_DES
-	      extern  char *crypt();
-#endif
-	      
+
 	      salt[0]=aconf->passwd[0];
 	      salt[1]=aconf->passwd[1];
 	      salt[2]='\0';
@@ -577,15 +567,15 @@ int	m_server(aClient *cptr, aClient *sptr, int parc, char *parv[])
 
 }
 
-int	m_server_estab(aClient *cptr)
+int
+m_server_estab(aClient *cptr)
 {
 	aClient   *acptr;
 	aConfItem *aconf, *bconf;
 	char      *inpath, *host, *s, *encr;
-	int       split, i;
+	int       i;
 
 	inpath = get_client_name(cptr,TRUE); /* "refresh" inpath with host */
-	split = mycmp(cptr->name, cptr->sockhost);
 	host = cptr->name;
 
 	current_load_data.conn_count++;
@@ -620,7 +610,6 @@ int	m_server_estab(aClient *cptr)
 	if(*cptr->passwd)
 	    {
 		char    salt[3];
-		extern  char *crypt();
 
 		salt[0]=aconf->passwd[0];
 		salt[1]=aconf->passwd[1];
@@ -733,14 +722,8 @@ int	m_server_estab(aClient *cptr)
 		if ((aconf = acptr->serv->nline) &&
 		    !match(my_name_for_link(me.name, aconf), cptr->name))
 			continue;
-		if (split) {
-			sendto_one(acptr,":%s SERVER %s 2 :[%s] %s",
-				   me.name, cptr->name,
-				   /*cptr->sockhost*/ inetntoa((char *)&cptr->ip), cptr->info);
-		}
-		else
-			sendto_one(acptr,":%s SERVER %s 2 :%s",
-				   me.name, cptr->name, cptr->info);
+		sendto_one(acptr,":%s SERVER %s 2 :%s",
+			me.name, cptr->name, cptr->info);
 	    }
 
 	/*
@@ -773,17 +756,9 @@ int	m_server_estab(aClient *cptr)
 			if (match(my_name_for_link(me.name, aconf),
 				    acptr->name) == 0)
 				continue;
-			split = (MyConnect(acptr) &&
-				 mycmp(acptr->name, acptr->sockhost));
-			if (split)
-				sendto_one(cptr, ":%s SERVER %s %d :[%s] %s",
-		   			   acptr->serv->up, acptr->name,
-					   acptr->hopcount+1,
-		   			   acptr->sockhost, acptr->info);
-			else
-				sendto_one(cptr, ":%s SERVER %s %d :%s",
-		   			   acptr->serv->up, acptr->name,
-					   acptr->hopcount+1, acptr->info);
+			sendto_one(cptr, ":%s SERVER %s %d :%s",
+		   		acptr->serv->up, acptr->name,
+				acptr->hopcount+1, acptr->info);
 		    }
 	    }
 
@@ -815,13 +790,6 @@ int	m_server_estab(aClient *cptr)
 
 			send_user_joins(cptr, acptr);
 		    }
-		else if (IsService(acptr))
-		    {
-			sendto_one(cptr,"NICK %s :%d",
-				   acptr->name, acptr->hopcount + 1);
-			sendto_one(cptr,":%s SERVICE * * :%s",
-				   acptr->name, acptr->info);
-		    }
 	    }
 	/*
 	** Last, pass all channels plus statuses
@@ -850,7 +818,8 @@ int	m_server_estab(aClient *cptr)
 **	parv[0] = sender prefix
 **	parv[1] = servername
 */
-int	m_info(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_info(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	char **text = infotext;
 
@@ -877,50 +846,6 @@ int	m_info(aClient *cptr, aClient *sptr, int parc, char *parv[])
 }
 
 
-/* Safe_info 
- *
- * Return a "safe" version of the server info reply
- * given a valid `acptr' pointing to a server
- *
- * Safe means: 1. Not null
- *             2. If showIp is 0 then the server's ip address won't
- *                be shown with the info.
- */
-const char* safe_info(int showIp, aClient* acptr)
-{
-	int i, ippfx;
-
-	if (!acptr) {
-		return "*Not On This Net*";
-	}
-	
-	if (!IsServer(acptr) && acptr->info && acptr->info[0])
-		return acptr->info;
-	
-	if (!acptr->info || !acptr->info[0]) {
-		return "(Unknown Location)";
-	}
-
-	if (!showIp && acptr->info[0] == '[') 
-	{
-		for(i = 1, ippfx = 0; acptr->info[i] && (i < 17); i++) {
-			if (acptr->info[i] == ']' && (i >= 7)) {
-				ippfx = 1;
-				break;
-			}
-			
-			if (acptr->info[i] != '.' && !isdigit(acptr->info[i]))
-				break;
-		}
-
-		if (ippfx) {
-			return acptr->info + i + 1;
-		}
-	}
-	
-	return acptr->info;
-}
-
 /*
 ** m_links
 **	parv[0] = sender prefix
@@ -930,7 +855,8 @@ const char* safe_info(int showIp, aClient* acptr)
 **	parv[1] = server to query 
 **      parv[2] = servername mask
 */
-int	m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	char *mask;
 	aClient *acptr;
@@ -957,7 +883,7 @@ int	m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
                
 		sendto_one(sptr, rpl_str(RPL_LINKS),
 			   me.name, parv[0], acptr->name, acptr->serv->up,
-			   acptr->hopcount, safe_info(IsOper(sptr), acptr));
+			   acptr->hopcount, acptr->info);
 	    }
 
 	sendto_one(sptr, rpl_str(RPL_ENDOFLINKS), me.name, parv[0],
@@ -988,13 +914,14 @@ int	m_links(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **            it--not reversed as in ircd.conf!
 */
 
-static int report_array[18][3] = {
+static int report_array[19][3] = {
 		{ CONF_CONNECT_SERVER,    RPL_STATSCLINE, 'C'},
 		{ CONF_NOCONNECT_SERVER,  RPL_STATSNLINE, 'N'},
 		{ CONF_CLIENT,            RPL_STATSILINE, 'I'},
 		{ CONF_AHURT,		  RPL_STATSKLINE, 'h'},
 		{ CONF_KILL,              RPL_STATSKLINE, 'K'},
 		{ CONF_ZAP,		  RPL_STATSKLINE, 'Z'},
+		{ CONF_SUP_ZAP,		  RPL_STATSKLINE, 'W'},
 		{ CONF_QUARANTINED_SERVER,RPL_STATSQLINE, 'q'},
 		{ CONF_QUARANTINED_NICK,  RPL_STATSQLINE, 'Q'},
 		{ CONF_LEAF,		  RPL_STATSLLINE, 'L'},
@@ -1003,7 +930,6 @@ static int report_array[18][3] = {
 		{ CONF_LOCOP,		  RPL_STATSOLINE, 'o'},
 		{ CONF_CRULEALL,	  RPL_STATSDLINE, 'D'},
 		{ CONF_CRULEAUTO,	  RPL_STATSDLINE, 'd'},
-		{ CONF_SERVICE,		  RPL_STATSSLINE, 'S'},
 		{ CONF_UWORLD,		  RPL_STATSULINE, 'U'},
 		{ CONF_MISSING,		  RPL_STATSXLINE, 'X'},
 		{ 0, 0, 0 }
@@ -1039,6 +965,9 @@ static	void	report_configured_links(aClient *sptr, int mask)
 			    tmp->tmpconf == KLINE_AKILL && !IsAnOper(sptr))
 			    continue;
 
+			if (tmp->status == CONF_SUP_ZAP && !IsAnOper(sptr))
+				continue;
+
 
                         if (tmp->status == CONF_OPERATOR ||
                             tmp->status == CONF_LOCOP    ||
@@ -1220,7 +1149,8 @@ char *get_client_namp(char *buf, aClient *acptr, int showports, int mask)
 }
 
 
-int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
       	static	char	Sformat[]  = ":%s %d %s SendQ SendM SendBytes RcveM RcveBytes Open_since :Idle";
 	static	char	Lformat[]  = ":%s %d %s %s %u %u %u %u %u %u :%u";
@@ -1304,11 +1234,10 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		report_configured_links(sptr, CONF_CLIENT);
 		break;
 	case 'K' : case 'k' :
-		if (!IsAnOper(sptr))
-			report_configured_links(sptr, CONF_KILL|CONF_ZAP);
-		else
-			report_configured_links(sptr, CONF_KILL|CONF_ZAP|CONF_AHURT);
-
+		if (IsAnOper(sptr))
+		{
+			report_configured_links(sptr, CONF_KILL|CONF_ZAP|CONF_AHURT|CONF_SUP_ZAP);
+		}
 		break;
 	case 'M' : case 'm' :
 		for (mptr = msgtab; mptr->cmd; mptr++)
@@ -1321,7 +1250,10 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		report_configured_links(sptr, CONF_OPS);
 		break;
 	case 'Q' : case 'q' :
-		report_configured_links(sptr, CONF_QUARANTINE);
+		if (IsAnOper(sptr))
+		{
+			report_configured_links(sptr, CONF_QUARANTINE);
+		}
 		break;
 	case 'R' : case 'r' :
 #ifdef DEBUGMODE
@@ -1334,9 +1266,6 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	case 'd' :
 		report_configured_links(sptr, CONF_CRULE);
 		break;
-	case 'S' : case 's' :
-		report_configured_links(sptr, CONF_SERVICE);
-		break;
 	case 'T' : case 't' :
 		tstats(sptr, parv[0]);
 		break;
@@ -1365,7 +1294,7 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		break;
 	case 'Z' : case 'z' :
 		if (IsAnOper(sptr)) { 
-			count_memory(sptr, parv[0]); 
+			count_memory(sptr, parv[0]);
 		}
 		break;
 	default :
@@ -1385,7 +1314,8 @@ int	m_stats(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[*] = parameters
 */
-int	m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	char	*para;
 
@@ -1399,7 +1329,7 @@ int	m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	** screen otherwise). Pass ERROR's from other sources to
 	** the local operator...
 	*/
-	if (IsPerson(cptr) || IsUnknown(cptr) || IsService(cptr))
+	if (IsPerson(cptr) || IsUnknown(cptr))
 		return 0;
 	if (cptr == sptr) {
 		sendto_serv_butone(&me, ":%s GLOBOPS :ERROR from %s -- %s",
@@ -1421,130 +1351,90 @@ int	m_error(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = optional message text
 */
-int	m_help(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_help(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	int i;
-        char    *message, *pv[4];
+        char    *message;
 
          if (check_registered(sptr))
                  return 0;
   
 	message = parc > 1 ? parv[1] : NULL;
 
-#if 0
-	if (BadPtr(message) && MyClient(cptr))
-	{
-	      if (parse_help(sptr, parv[0], "INDEX") < 0)
-	           sendto_one(sptr, ":%s NOTICE %s :!ERROR! Help Database is Missing!", me.name, parv[0]);
-	      return 0;
-	}
-#endif
-
-	if (BadPtr(message) || !mycmp(message, "msgtab"))
-	{
- 	   for (i = 0; msgtab[i].cmd; i++)
-               if ((!IsHurt(sptr)||(msgtab[i].while_hurt>0))&&!(msgtab[i].flags & MF_H))
-		   sendto_one(sptr,":%s NOTICE %s :%s",
-			      me.name, parv[0], msgtab[i].cmd);
-	   return 0;
-	}
-        if (!mycmp(message, "ignore") && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr)))
-        {
-            if (!h_nignores)
-                  sendto_one(sptr, ":%s NOTICE %s :There are no help ignores.", me.name, parv[0]);
-            for ( i = 0 ; i < h_nignores; i++ )
-                  sendto_one(sptr, ":%s NOTICE %s :\2%2d\2. %s", me.name, parv[0], i+1, h_ignores[i]);
-            return(0);
-        }
-        if (!myncmp(message, "+ignore ", 7) && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr)))
-        {
-            if (BadPtr(message+7) || BadPtr(message+8))
-            {
-                sendto_one(sptr, ":%s NOTICE %s :Ignore WHAT?", me.name, parv[0]);
-                return(0);
-            }
-            if (!match(message+8, ".!......@...")     || 
-                !match(message+8, ".!......@.....")   || 
-                !match(message+8, ".!......@...net")  ||
-                !match(message+8, ".!......@...com")  ||
-                !match(message+8, ".!......@...org"))
-            {
-                sendto_one(sptr, ":%s NOTICE %s :That is not a legal ignore HelpOp mask.", me.name, parv[0]);
-                if (IsServer(cptr))
-                    sendto_serv_butone(cptr, ":%s HELP %s", parv[0], message);
-                return (0);
-            }
-            helpop_ignore(message + 8);
-            if (!IsServer(cptr))
-                sendto_one(sptr, ":%s NOTICE %s :Ignore for \2%s\2 set.", me.name, parv[0], message+8);
-            else if (IsServer(sptr))
-            {
-                sendto_serv_butone(cptr, ":%s HELP %s", parv[0], message);
-                return (0); /* Don't send helpops for Net.burst */
-            }
-        }
-        if (!myncmp(message, "-ignore ", 7) && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr)))
-        {
-            for ( i = 0 ; i < h_nignores; i++ )
-                  if (!mycmp(message+8, h_ignores[i]))
-                      break;
-            if ( i >= h_nignores )
-            {
-                  if (MyClient(sptr))
-                      sendto_one(sptr, ":%s NOTICE %s :No such ignore.", me.name, parv[0]);
-                  return (0);
-            }
-            if (!IsServer(cptr))
-                sendto_one(sptr, ":%s NOTICE %s :Ignore for \2%s\2 removed.", me.name, parv[0], h_ignores[i]);
-            helpop_unignore(i);
-        }
-        if ((!myncmp(message, "-ignore ", 7) || !myncmp(message, "+ignore ", 7)
-            || !mycmp(message, "ignore")) && !(IsOper(sptr) || IsHelpOp(sptr) || IsServer(sptr) ))
-        {
-            sendto_one(sptr, ":%s NOTICE %s :Permission denied -- You must be a helpop to edit help system ignores.", me.name, parv[0]);
-            return (0);
-        }
-
-#if 0
-	if (!mycmp(message, "topics"))
-	{
-                 int shown = 0;
-		 char tstr[8192] = ""; 
-		 sendto_one(sptr, ":%s NOTICE %s :--- Help Topics ---", me.name, sptr->name);
-		 #define FLUSH_TSTR { \
-			if (tstr[0]) \
-				sendto_one(sptr, ":%s NOTICE %s :%s", me.name, sptr->name, tstr); \
-				tstr[0] = '\0'; \
-				    }
-		for (i = 0; i<nhtopics; i++)
-		{
-			if ((++shown)%2 == 0)
-			    FLUSH_TSTR;
-			sprintf(tstr, "%s          %-20s", tstr, htopics[i]->command);
-		}
-		FLUSH_TSTR;
-		sendto_one(sptr, ":%s NOTICE %s :--- End of List ---", me.name, sptr->name);
-		#undef FLUSH_TSTR
+	if (BadPtr(message) || !mycmp(message, "msgtab")) {
+		for (i = 0; msgtab[i].cmd; i++)
+			if ((!IsHurt(sptr) || (msgtab[i].while_hurt > 0))
+			    && !(msgtab[i].flags & MF_H))
+				sendto_one(sptr,":%s NOTICE %s :%s",
+					   me.name, parv[0], msgtab[i].cmd);
 		return 0;
 	}
-#endif        
+        if (!mycmp(message, "ignore")
+	    && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr))) {
+		if (!h_nignores)
+			sendto_one(sptr, ":%s NOTICE %s :There are no help ignores.", me.name, parv[0]);
+		for ( i = 0 ; i < h_nignores ; i++ )
+			sendto_one(sptr, ":%s NOTICE %s :\2%2d\2. %s",
+				   me.name, parv[0], i+1, h_ignores[i]);
+		return (0);
+        }
+        if (!myncmp(message, "+ignore ", 7)
+	    && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr))) {
+		if (BadPtr(message+7) || BadPtr(message+8)) {
+			sendto_one(sptr, ":%s NOTICE %s :Ignore WHAT?",
+				   me.name, parv[0]);
+			return(0);
+		}
+		if (!match(message+8, ".!......@...")     || 
+		    !match(message+8, ".!......@.....")   || 
+		    !match(message+8, ".!......@...net")  ||
+		    !match(message+8, ".!......@...com")  ||
+		    !match(message+8, ".!......@...org")) {
+			sendto_one(sptr, ":%s NOTICE %s :That is not a legal ignore HelpOp mask.", me.name, parv[0]);
+			if (IsServer(cptr))
+				sendto_serv_butone(cptr, ":%s HELP %s", parv[0], message);
+			return (0);
+		}
+		helpop_ignore(message + 8);
+		if (!IsServer(cptr))
+			sendto_one(sptr, ":%s NOTICE %s :Ignore for \2%s\2 set.", me.name, parv[0], message+8);
+		else if (IsServer(sptr)) {
+			sendto_serv_butone(cptr, ":%s HELP %s",
+					   parv[0], message);
+			return (0); /* Don't send helpops for Net.burst */
+		}
+        }
+        if (!myncmp(message, "-ignore ", 7)
+	    && (IsHelpOp(sptr) || IsOper(sptr) || IsServer(sptr))) {
+		for ( i = 0 ; i < h_nignores ; i++ )
+			if (!mycmp(message+8, h_ignores[i]))
+				break;
+		if (i >= h_nignores) {
+			if (MyClient(sptr))
+				sendto_one(sptr, ":%s NOTICE %s :No such ignore.", me.name, parv[0]);
+			return (0);
+		}
+		if (!IsServer(cptr))
+			sendto_one(sptr, ":%s NOTICE %s :Ignore for \2%s\2 removed.", me.name, parv[0], h_ignores[i]);
+		helpop_unignore(i);
+        }
+        if ((!myncmp(message, "-ignore ", 7)
+	     || !myncmp(message, "+ignore ", 7)
+	     || !mycmp(message, "ignore"))
+	    && !(IsOper(sptr) || IsHelpOp(sptr) || IsServer(sptr) )) {
+		sendto_one(sptr, ":%s NOTICE %s :Permission denied -- You must be a helpop to edit help system ignores.", me.name, parv[0]);
+		return (0);
+        }
 
 /* Drags along from wallops code... I'm not sure what it's supposed to do, 
    at least it won't do that gracefully, whatever it is it does - but
    checking whether or not it's a person _is_ good... -Donwulff */
 
-	if (!IsServer(sptr) && MyConnect(sptr) && !IsPerson(sptr))
-	    {
+	if (!IsServer(sptr) && MyConnect(sptr) && !IsPerson(sptr)) {
 		check_registered(sptr);
 		return 0;
-	    }
-
-/*	if (IsServer(sptr) || IsHelpOp(sptr)) {
- *		sendto_serv_butone(IsServer(cptr) ? cptr : NULL,
- *				   ":%s HELP %s", parv[0], message);
- *		sendto_helpops("from %s (HelpOp): %s", parv[0], message);
- *	}
- */
+	}
 
         if (IsServer(cptr) /*|| IsHelpOp(sptr)*/ ) {
                 sendto_serv_butone(IsServer(cptr) ? cptr : NULL,
@@ -1573,8 +1463,7 @@ int	m_help(aClient *cptr, aClient *sptr, int parc, char *parv[])
                 sendto_helpops("from %s%s: %s", parv[0], IsHelpOp(sptr) ? " (HelpOp)" : MyClient(sptr) ? " (Local)" : "", message+adj);
                 sendto_serv_butone(cptr, ":%s HELP %s", parv[0], message+adj);
         } else if (MyClient(sptr) && parse_help(sptr, parv[0], message) > 0) {
-        /*} else if (MyClient(sptr) && index(message, '?') && parse_help(sptr, parv[0], (char *)interpret_question(message)) > 0) {*/
-        } else if (!nohelp_message(sptr)) {
+        } else if (!nohelp_message(sptr, 0)) {
         } else if (MyConnect(sptr)) {
                 sendto_serv_butone(IsServer(cptr) ? cptr : NULL,
                                    ":%s HELP %s", parv[0], message);
@@ -1617,7 +1506,7 @@ int	 m_lusers(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	(void)collapse(parv[1]);
 	for (acptr = client; acptr; acptr = acptr->next)
 	    {
-		if (parc>1)
+		    if (parc>1) {
 			if (!IsServer(acptr) && acptr->user)
 			    {
 				if (match(parv[1], acptr->user->server))
@@ -1626,6 +1515,7 @@ int	 m_lusers(aClient *cptr, aClient *sptr, int parc, char *parv[])
 			else
 	      			if (match(parv[1], acptr->name))
 					continue;
+		    }
 
 		switch (acptr->status)
 		{
@@ -1695,7 +1585,8 @@ int	 m_lusers(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[2] = port number
 **	parv[3] = remote server
 */
-int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	int	port, tmpport, retval;
 	aConfItem *aconf, *cconf;
@@ -1755,8 +1646,8 @@ int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	if (!aconf)
 	    {
 	      sendto_one(sptr,
-			 "NOTICE %s :Connect: Host %s not listed in irc.conf",
-			 parv[0], parv[1]);
+			 ":%s NOTICE %s :Connect: Host %s not listed in %s",
+			 me.name, parv[0], parv[1], CPATH);
 	      return 0;
 	    }
 	/*
@@ -1770,8 +1661,8 @@ int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		if ((port = atoi(parv[2])) <= 0)
 		    {
 			sendto_one(sptr,
-				   "NOTICE %s :Connect: Illegal port number",
-				   parv[0]);
+				   ":%s NOTICE %s :Connect: Illegal port number",
+				   me.name, parv[0]);
 			return 0;
 		    }
 	    }
@@ -1794,8 +1685,8 @@ int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    if (crule_eval (cconf->passwd))
 	      {
 		sendto_one(sptr,
-			   "NOTICE %s :Connect: Disallowed by rule: %s",
-			   parv[0], cconf->name);
+			   ":%s NOTICE %s :Connect: Disallowed by rule: %s",
+			   me.name, parv[0], cconf->name);
 		return 0;
 	      }
 
@@ -1842,9 +1733,10 @@ int	m_connect(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = message text
 */
-int	m_wallops(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_wallops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
-	char	*message, *pv[4];
+	char	*message;
 
          if (check_registered(sptr))
                  return 0;
@@ -1872,9 +1764,10 @@ int	m_wallops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = message text
 */
-int	m_gnotice(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_gnotice(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
-	char *message, *pv[4];
+	char *message;
 
          if (check_registered(sptr))
                  return 0;
@@ -1894,7 +1787,7 @@ int	m_gnotice(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    }
 	sendto_serv_butone(IsServer(cptr) ? cptr : NULL, ":%s GNOTICE :%s",
 		 parv[0], message);
-	sendto_failops_whoare_opers("from %s: %s", parv[0], message);
+	sendto_failops("from %s: %s", parv[0], message);
 	return 0;
 }
 
@@ -1903,9 +1796,10 @@ int	m_gnotice(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **      parv[0] = sender prefix
 **      parv[1] = message text
 */
-int     m_globops(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_globops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
-        char    *message, *pv[4];
+        char    *message;
 
          if (check_registered(sptr))
                  return 0;
@@ -1925,7 +1819,7 @@ int     m_globops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	    }
         sendto_serv_butone(IsServer(cptr) ? cptr : NULL,
                         ":%s GLOBOPS :%s", parv[0], message);
-        sendto_failops_whoare_opers("from %s: %s", parv[0], message);
+        sendto_failops("from %s: %s", parv[0], message);
         return 0;
 }
 
@@ -1934,9 +1828,10 @@ int     m_globops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **      parv[0] = sender prefix
 **      parv[1] = message text
 */
-int     m_locops(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_locops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
-        char    *message, *pv[4];
+        char    *message;
 
         if (check_registered_user(cptr))
                 return 0;
@@ -1965,27 +1860,24 @@ int     m_locops(aClient *cptr, aClient *sptr, int parc, char *parv[])
 */
 int     m_goper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
-        char *message, *pv[4];
+        char *message;
 
-         if (check_registered(sptr))
+	if (check_registered(sptr))
                  return 0;
  
-       message = parc > 1 ? parv[1] : NULL;
+	message = parc > 1 ? parv[1] : NULL;
 
-        if (BadPtr(message))
-            {
+        if (BadPtr(message)) {
                 sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS),
                            me.name, parv[0], "GOPER");
                 return 0;
-            }
-/*      if (!IsServer(sptr) && MyConnect(sptr) && !IsAnOper(sptr))*/
-	if (!IsServer(sptr) || !IsULine(cptr,sptr))
-	    {
+	}
+	if (!IsServer(sptr) || !IsULine(cptr,sptr)) {
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name, parv[0]);
 		return 0;
-	    }
+	}
 	sendto_serv_butone(IsServer(cptr) ? cptr : NULL, ":%s GOPER :%s", 
-		parv[0], message);
+			   parv[0], message);
         sendto_opers("from %s: %s", parv[0], message);
 	return 0;
 }
@@ -1994,7 +1886,8 @@ int     m_goper(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = servername
 */
-int	m_time(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_time(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	if (check_registered_user(sptr))
 		return 0;
@@ -2010,7 +1903,8 @@ int	m_time(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = servername
 */
-int	m_admin(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_admin(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	aConfItem *aconf;
 
@@ -2044,7 +1938,8 @@ int	m_admin(aClient *cptr, aClient *sptr, int parc, char *parv[])
 ** m_rehash
 **
 */
-int	m_rehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_rehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	if (!MyClient(sptr) || !OPCanRehash(sptr))
 	    {
@@ -2065,7 +1960,8 @@ int	m_rehash(aClient *cptr, aClient *sptr, int parc, char *parv[])
 ** parv[1] - reason for restart (optional)
 **
 */
-int	m_restart(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_restart(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
          if (check_registered(sptr))
                  return 0;
@@ -2090,7 +1986,8 @@ int	m_restart(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = servername
 */
-int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	int	i;
 	aClient	*acptr;
@@ -2134,20 +2031,16 @@ int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	wilds = !parv[1] || index(tname, '*') || index(tname, '?');
 	dow = wilds || doall;
 
-#ifndef _WIN32
 	for (i = 0; i < MAXCONNECTIONS; i++)
 		link_s[i] = 0, link_u[i] = 0;
-#else
-	bzero(link_s, sizeof(link_s));
-	bzero(link_u, sizeof(link_u));
-#endif
 
-	if (doall)
+	if (doall) {
 		for (acptr = client; acptr; acptr = acptr->next)
 			if (IsPerson(acptr))
 				link_u[acptr->from->fd]++;
 			else if (IsServer(acptr))
 				link_s[acptr->from->fd]++;
+	}
 
 	/* report all direct connections */
 	
@@ -2237,16 +2130,6 @@ int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 					   now - acptr->lasttime);
 			cnt++;
 			break;
-		case STAT_SERVICE:
-			sendto_one(sptr, rpl_str(RPL_TRACESERVICE),
-				   me.name, parv[0], class, name);
-			cnt++;
-			break;
-		case STAT_LOG:
-			sendto_one(sptr, rpl_str(RPL_TRACELOG), me.name,
-				   parv[0], LOGFILE, acptr->port);
-			cnt++;
-			break;
 		default: /* ...we actually shouldn't come here... --msa */
 			sendto_one(sptr, rpl_str(RPL_TRACENEWTYPE), me.name,
 				   parv[0], name);
@@ -2282,7 +2165,8 @@ int	m_trace(aClient *cptr, aClient *sptr, int parc, char *parv[])
 **	parv[0] = sender prefix
 **	parv[1] = servername
 */
-int	m_motd(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_motd(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	int	fd, nr;
 	char	line[80];
@@ -2332,7 +2216,8 @@ int	m_motd(aClient *cptr, aClient *sptr, int parc, char *parv[])
 /*
 ** m_close - added by Darren Reed Jul 13 1992.
 */
-int	m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	aClient	*acptr;
 	int	i;
@@ -2354,6 +2239,18 @@ int	m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
 		if (!IsUnknown(acptr) && !IsConnecting(acptr) &&
 		    !IsHandshake(acptr))
 			continue;
+		if (parc >= 2 && match(parv[1], acptr->sockhost)) {
+			continue;
+		}
+
+		if (parc <= 1) {
+			sendto_one(sptr, ":%s NOTICE %s :%d. Unknown [%s.%d] [n:%s] [%s]", me.name,
+				       	sptr->name, i, acptr->sockhost, acptr->port,
+				       	BadPtr(acptr->name) ? "" : acptr->name,
+					DoingDNS(acptr) ? "DoingDNS" : "");
+			continue;
+		}
+		
 		sendto_one(sptr, rpl_str(RPL_CLOSING), me.name, parv[0],
 			   get_client_name(acptr, TRUE), acptr->status);
 		(void)exit_client(acptr, acptr, acptr, "Oper Closing");
@@ -2363,7 +2260,8 @@ int	m_close(aClient *cptr, aClient *sptr, int parc, char *parv[])
 	return 0;
 }
 
-int	m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
+int
+m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
 {
 	aClient	*acptr;
 	int	i, nomatch = 0;
@@ -2419,36 +2317,10 @@ int	m_die(aClient *cptr, aClient *sptr, int parc, char *parv[])
 				   me.name, get_client_name(sptr, TRUE),
                                    parc > 2 ? parv[2] : "No reason specified");
 	    }
-	(void)s_die();
+	(void)s_die(0);
 	return 0;
 }
 
-#ifdef _WIN32
-/*
- * Added to let the local console shutdown the server without just
- * calling exit(-1), in Windows mode.  -Cabal95
- */
-int	localdie(void)
-{
-	aClient	*acptr;
-	int	i;
-
-	for (i = 0; i <= highest_fd; i++)
-	    {
-		if (!(acptr = local[i]))
-			continue;
-		if (IsClient(acptr))
-			sendto_one(acptr,
-                ":%s NOTICE %s :Server Terminated by local console",
-                me.name, acptr->name);
-		else if (IsServer(acptr))
-			sendto_one(acptr, ":%s ERROR :Terminated by local console",
-                me.name);
-	    }
-	(void)s_die();
-	return 0;
-}
-#endif
 
 /*
 ** m_showmask                  Show what a hostname would be, masked
@@ -2472,6 +2344,8 @@ int     m_showmask(aClient *cptr, aClient *sptr, int parc, char *parv[])
   strncpyzt(mask, parv[1], sizeof(mask));
   sendto_one(sptr, ":%s NOTICE %s :%25s -> %25s", me.name, parv[0],
              parv[1], genHostMask(mask));
+
+  return 0;
 }
 
 
@@ -2504,6 +2378,11 @@ int   m_watch(aClient *cptr, aClient *sptr, int parc, char *parv[])
         char  *s, *p, *user;
         char def[2] = "l";
 
+	if (check_registered_user(sptr))
+	{
+		return 0;
+	}
+
         if (parc < 2)
         {
                 /* Default to 'l' - list who's currently online */
