@@ -27,11 +27,17 @@
 #include <sys/stat.h>
 #include "h.h"
 
+#include "ircd/send.h"
+#include "ircd/string.h"
+
+IRCD_RCSID("$Id: s_service.c,v 1.1.1.7.2.3 2005/01/07 21:14:34 onno Exp $");
 
 /* these are only considered services when the nicks are U-lined. */
 /* this array is an evil hack so pointer comparisons will do... */
-const char *service_nick[] = { "NickServ", "ChanServ", "MemoServ", "OperServ", 
-                               "InfoServ", "GameServ", (char *)0, (char *)0, (char *)0 };
+const char *service_nick[] = {
+	"NickServ", "ChanServ", "MemoServ", "OperServ", 
+	"InfoServ", "GameServ", (char *)0, (char *)0, (char *)0
+};
 
 extern __inline int    m_sendto_service();
 
@@ -41,6 +47,11 @@ int   m_services(aClient *cptr, aClient *sptr, int parc, char *parv[])
      const char *nickchan;
      aClient *acptr;
 
+	if (check_registered_user(sptr))
+	{
+		return 0;
+	}
+
      /* /services <command> <nick|chan> <args> */
      if ( parc < 3 && (parc<2 || ( strcasecmp(parv[1], "help") && strcasecmp(parv[1], "info") )) )
      {
@@ -74,7 +85,13 @@ int   m_identify(aClient *cptr, aClient *sptr, int parc, char *parv[])
      aClient *acptr;
      const char *sService;
      const char *nickchan, *pass;
-     /* : /identify <nick|chan> <pass> */
+
+ 	if (check_registered_user(sptr))
+	{
+		return 0;
+	}
+
+    /* : /identify <nick|chan> <pass> */
      if ( parc < 2 )
      {
              sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "IDENTIFY");
@@ -151,21 +168,8 @@ const	char *sService;
        {
                if (fOper && !IsAnOper(sptr) && MyClient(sptr))
                   return -2;
-#if 0
-               if (!MyClient(sptr) || myncmp(parv[1], "help ", 5) || BadPtr(parv[1]+5))
-#endif
                    sendto_one(acptr,":%s PRIVMSG %s@%s :%s", parv[0],
                            sService, SERVICES_NAME, parv[1]);
-#if 0
-               else if (MyClient(sptr))
-               {
-                   char buf[2048] = "";
-                   sprintf(buf, "%s %s", sService, parv[1]+5);
-                   if (parse_help(sptr, parv[0], buf) < 1)
-                       sendto_one(acptr,":%s PRIVMSG %s@%s :%s", parv[0],
-                                  sService, SERVICES_NAME, parv[1]);
-               }
-#endif
        }
        else
                sendto_one(sptr, err_str(ERR_SERVICESDOWN), me.name,
